@@ -8,7 +8,7 @@
 
 package Log::Report;
 use vars '$VERSION';
-$VERSION = '1.30';
+$VERSION = '1.32';
 
 use base 'Exporter';
 
@@ -65,7 +65,7 @@ sub report($@)
     my ($reason, $message) = (shift, shift);
 
     my $stop = exists $opts->{is_fatal} ? $opts->{is_fatal} : is_fatal $reason;
-    my $try  = $nested_tries[-1];
+    my $try  = $nested_tries[-1];  # WARNING: overloaded boolean, use 'defined'
 
     my @disp;
     if(defined $try)
@@ -94,6 +94,7 @@ sub report($@)
         else
         {   @disp    = grep $_->name eq $to, @disp;
         }
+        push @disp, $try if defined $try;
 
         @disp || $stop
             or return;
@@ -134,7 +135,7 @@ sub report($@)
 
     if(my $disp_name = $message->to)
     {   @disp = grep $_->name eq $disp_name, @disp;
-        push @disp, $try if $try && $disp_name ne 'try';
+        push @disp, $try if defined $try && $disp_name ne 'try';
         @disp or return;
     }
 
