@@ -8,7 +8,7 @@
 
 package Log::Report::Dispatcher::Try;
 use vars '$VERSION';
-$VERSION = '1.30';
+$VERSION = '1.32';
 
 use base 'Log::Report::Dispatcher';
 
@@ -49,16 +49,13 @@ sub died(;$)
 sub exceptions() { @{shift->{exceptions}} }
 
 
-sub hides($)
-{   my $h = shift->{hides} or return 0;
-    keys %$h ? $h->{(shift)} : 1;
-}
+sub hides($) { $_[0]->{LRDT_hides}{$_[1]} }
 
 
 sub hide(@)
 {   my $self = shift;
     my @reasons = expand_reasons(@_ > 1 ? \@_ : shift);
-    $self->{hides} = +{ map +($_ => 1), @reasons };
+    $self->{LRDT_hides} = +{ map +($_ => 1), @reasons };
 }
 
 
@@ -103,7 +100,6 @@ sub failed()  {   defined shift->{died}}
 sub success() { ! defined shift->{died}}
 
 
-
 sub wasFatal(@)
 {   my ($self, %args) = @_;
     defined $self->{died} or return ();
